package de.srsoftware.belegscanner.gui;

import de.srsoftware.belegscanner.Worker;
import de.srsoftware.belegscanner.listeners.ProjectListener;
import de.srsoftware.belegscanner.model.ScanProject;
import de.srsoftware.belegscanner.model.api.Project;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.Border;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;

import static de.srsoftware.belegscanner.Application.t;
import static java.awt.Color.black;
import static javax.swing.BorderFactory.createCompoundBorder;
import static javax.swing.BorderFactory.createEmptyBorder;
import static javax.swing.BorderFactory.createLineBorder;
import static javax.swing.BorderFactory.createMatteBorder;

/**
 * 
 * Creates a table of previously scanned documents.
 * @author Stephan Richter <s.richter@srsoftware.de>
 *
 */
public class DocTable extends JPanel{
	private static final Border BOTTOM_LINE = createMatteBorder(0, 0, 1, 0, black);
	private static final Border IMAGE_BUTTON = createCompoundBorder(createLineBorder(black),createEmptyBorder(5,5,5,5));
	private static final Logger LOG = LoggerFactory.getLogger(DocTable.class);
	private static final int IMAGE_HEIGHT = 32;
	private static FilenameFilter PAGES = (dir,name) -> name.toLowerCase().endsWith("pdf") && name.toLowerCase().contains("page");
	private static int rowCounter = 0;
	private static Font btnFont = new Font("Arial", Font.PLAIN, 28);
	private final Worker worker;
	private Image openDir, joinFiles,preview,finish,trash;

	private List<Project> projects = new ArrayList<>();

	private List<ProjectListener> listeners = new ArrayList<>();

	private static final long serialVersionUID = 1073955198529023744L;
	
	private GridBagConstraints constraints;
	private final HashMap<String,Row> rows = new HashMap<>();

	private class Row{

		private JLabel status;
		private JLabel pathLabel;
		private JPanel buttons;
		private Project project;
		private JButton dropButton,finishButton,joinButton,previewButton;

		public Row(Project project) {
			this.project = project;
			rows.put(project.pattern(), this);
			constraints.gridy = ++rowCounter;
			constraints.gridx = 0;
			add(pathLabel = new JLabel(project.pattern()), constraints);
			constraints.gridx = 1;
			add(status = new JLabel("neu"),constraints);

			buttons = new JPanel();
			buttons.setLayout(new FlowLayout());

			buttons.add(button(openDir,t("open folder"),ev -> project.openDir()));
			buttons.add(joinButton = button(joinFiles,t("join PDFs"),ev -> worker.join(project)));
			buttons.add(previewButton = button(preview,t("display preview"),ev -> worker.preview(project.lastPage())));
			buttons.add(dropButton = button(trash,t("drop last page"),ev -> worker.preview(project.dropLastPage().lastPage())));
			buttons.add(finishButton = button(finish,t("done"),ev -> drop(this)));

			constraints.gridx = 2;
			enableButtons(false);
			add(buttons,constraints);
		}

		/**
		 * drop the selected document from the table
		 * @param row
		 */
		private void drop(Row row) {
			if (row.joinButton.isEnabled()) project.join();
			remove(pathLabel);
			remove(status);
			remove(buttons);
			invalidate();
			repaint();
			rows.remove(project.pattern());
			projects.remove(project);
		}

		protected void enableButtons(boolean enable){
			dropButton.setEnabled(enable);
			joinButton.setEnabled(enable);
			previewButton.setEnabled(enable);
			finishButton.setEnabled(enable);
		}

		/**
		 * update status for selected document
		 * @param status
		 * @return
		 */
		public Row status(String status) {
			this.status.setText(status);
			return this;
		}

		/**
		 * remove the join button assigned with the selected document
		 */
		public void setJoinButton(boolean enabled) {
			joinButton.setEnabled(enabled);
			joinButton.setVisible(enabled);
		}
	}

	/**
	 * create a new DocTable
	 */
	public DocTable(Worker worker) {
		setLayout(new GridBagLayout());
		this.worker = worker;
		worker.setDocTable(this);
		constraints = new GridBagConstraints();
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.insets = new Insets(3, 10, 3, 10);
		constraints.gridy = 0;
		constraints.gridx = 0;
		JLabel ordner = new JLabel(t("Directory"),SwingConstants.CENTER);
		ordner.setBorder(BOTTOM_LINE);
		add(ordner,constraints);
		constraints.gridx = 1;
		JLabel status = new JLabel(t("State"),SwingConstants.CENTER);
		status.setBorder(BOTTOM_LINE);
		add(status,constraints);
		constraints.gridx = 2;
		JLabel actions = new JLabel(t("Actions"),SwingConstants.CENTER);
		actions.setBorder(BOTTOM_LINE);
		add(actions,constraints);

		openDir = load("open-dir.png").orElse(null);
		joinFiles = load("join.png").orElse(null);
		preview = load("preview.png").orElse(null);
		finish = load("finish.png").orElse(null);
		trash = load("trash.png").orElse(null);
	}

	private static Optional<Image> load(String name){
		try {
			var path = String.join("/","","img",name);
			BufferedImage large = ImageIO.read(DocTable.class.getResource(path));
			double height = large.getHeight();
			var width = large.getWidth();
			double factor = IMAGE_HEIGHT / height;
			return Optional.of(large.getScaledInstance((int)(width*factor),IMAGE_HEIGHT,Image.SCALE_SMOOTH));
		} catch (IOException e) {
			return Optional.empty();
		}
	}

	private static JButton button(Image image, String tooltip, ActionListener listener) {
		JButton btn = new JButton();
		if (image != null) btn.setIcon(new ImageIcon(image));
		btn.setToolTipText(tooltip);
		btn.addActionListener(listener);
		btn.setBorder(IMAGE_BUTTON);
		return btn;
	}

	public Project createProject(String resolvedPath) {
		var project = new ScanProject(resolvedPath);
		new Row(project);
		projects.add(project);
		return project;
	}
	public Optional<Project> getProject(String resolvedPath) {
		return projects.stream().filter(p -> resolvedPath.equals(p.pattern())).findAny();
	}

	public void setState(Project project, String status) {
		getRow(project).ifPresent(row -> row.status.setText(status));
	}

	public void enableButtons(Project project, boolean enable) {
		getRow(project).ifPresent(row -> row.enableButtons(enable));
	}

	private Optional<Row> getRow(Project project) {
		return Optional.ofNullable(rows.get(project.pattern()));
	}
}
