package de.srsoftware.belegscanner.gui;

import de.srsoftware.belegscanner.Worker;
import de.srsoftware.tools.gui.DateChooser;
import de.srsoftware.tools.gui.SelectComboBox;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.util.*;
import java.util.List;
import java.util.stream.Stream;

import static de.srsoftware.belegscanner.Application.t;
import static javax.swing.BoxLayout.X_AXIS;
import static javax.swing.BoxLayout.Y_AXIS;

/**
 * create the application's toolbar
 * @author Stephan Richter <s.richter@srsoftware.de>
 *
 */
public class Toolbar extends JPanel {

	private final SelectComboBox pathPicker, profilePicker;
	private static final int OFFSET = 2;

	public class AdditionalInput {
		private final JPanel panel;
		private final SelectComboBox input;
		private boolean required;

		AdditionalInput(JPanel panel, SelectComboBox input, boolean required){
			this.panel = panel;
			this.input = input;
			this.required = required;
		}

		public void notRequired () {
			required = false;
		}
	}
	private static final Logger LOG = LoggerFactory.getLogger(Toolbar.class);
	private final Worker worker;
	private static final long serialVersionUID = -5834326573752788233L;
	private static EmptyBorder BORDER = new EmptyBorder(0,5,5,5);
	private static Map<String, AdditionalInput> additionalInputs = new HashMap<>();

	private Vector<Object> paths = new Vector<>();


	private DateChooser date;


	private JButton scanButton;

	public Toolbar(Worker worker) {
		this.worker = worker;
		worker.setToolbar(this);
		setLayout(new BoxLayout(this, Y_AXIS));
		add(datePicker());
		add(input(t("Profile"), profilePicker = profilePicker()));
		add(input(t("Path"),pathPicker = pathPicker()));
		add(new FormatSelector(worker));
		add(new ResolutionSelector(worker));
		add(new ImproveSelector(worker));
		add(new RotationSelector(worker));
		add(new TypeSelector(worker));
		var buttonBox = new JPanel();
		buttonBox.setLayout(new BoxLayout(buttonBox,X_AXIS));
		buttonBox.add(scanButton());
		buttonBox.add(Box.createRigidArea(new Dimension(10,10)));
		buttonBox.add(helpButton());
		add(buttonBox);
		add(Box.createVerticalGlue());
		Arrays.stream(getComponents()).filter(c -> c instanceof JPanel).map(JPanel.class::cast).forEach(p -> p.setBorder(BORDER));
	}

	public Stream<Map.Entry<String, String>> additionalFields() {
		return additionalInputs.entrySet()
				.stream()
				.filter(entry -> entry.getValue().required)
				.map(entry -> new AbstractMap.SimpleEntry<>(entry.getKey(),entry.getValue().input.getText()));
	}

	private SelectComboBox profilePicker() {
		return new SelectComboBox(worker.getProfiles().keySet()).onUpdateText(worker::loadProfile).onDelete(worker::dropProfile);
	}

	/**
	 * create the date picker
	 * @return
	 */
	private Component datePicker() {
		date = new DateChooser();
		Dimension dim = new Dimension(400, 250);
		date.setMaximumSize(dim);
		worker.setDatePicker(date);

		return date;
	}

	public void dropUnusedAdditionalFields() {
		additionalInputs.entrySet()
				.stream()
				.filter(entry -> !entry.getValue().required)
				.map(Map.Entry::getValue)
				.map(ai -> ai.panel)
				.forEach(this::remove);
	}

	/**
	 * create help button
	 * @return
	 */
	private Component helpButton() {
		var button = new JButton(t("help"));
		button.addActionListener(ev -> worker.showHelp());
		return button;
	}

	/**
	 * creates a labeled input field
	 * @param caption
	 * @param component
	 * @return
	 */
	private JPanel input(String caption, JComponent component) {
		component.setMaximumSize(new Dimension(600, 40));
		component.setToolTipText(t("Press Shift + Delete to remove an entry."));
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, X_AXIS));
		panel.add(new JLabel(caption+": "));
		panel.add(component);
		return panel;
	}
	
	/**
	 * creates an auto-complete text field for setting the scan target path
	 * @return
	 */
	private SelectComboBox pathPicker() {
		JSONObject profiles = worker.getProfiles();
		for (String profileName : profiles.keySet()) {
			JSONObject profileInfo = profiles.getJSONObject(profileName);
			if (profileInfo.has("path")) paths.add(profileInfo.get("path"));
		}
		paths.add("$"+t("HOME")+"/$"+t("YEAR")+"/$"+t("MONTH")+"-$"+t("DAY")+" - $"+t("PROFILE")+"/$"+t("EXTRA")+" - "+t("Page")+" $"+t("PAGE"));
		return worker.setPathPicker(new SelectComboBox(paths));
	}


	public String requireField(String varName){
		LOG.debug("requireField({})",varName);
		var additionalInput = additionalInputs.get(varName);
		if (additionalInput == null) {
			LOG.debug("Field missing, creating…");
			var input = new SelectComboBox(List.of());
			input.onUpdateText(newText -> worker.resolvePath());
			JPanel panel = input(varName,input);
			additionalInputs.put(varName, additionalInput = new AdditionalInput(panel,input,true));
			input.setBorder(BORDER);
			add(panel,getComponentCount()-OFFSET);
		} else {
			LOG.debug("Field preset, flagging as required");
			additionalInput.required = true;
			if (!Arrays.asList(getComponents()).contains(additionalInput.panel)){
				add(additionalInput.panel,getComponentCount()-OFFSET);
			}
		}
		additionalInput.input.setElements(worker.getSamples(varName));
		return additionalInput.input.getText();
	}

	public void resetAdditionalFields() {
		additionalInputs.values().forEach(AdditionalInput::notRequired);
	}


	/**
	 * create scan button
	 * @return
	 */
	private Component scanButton() {
		scanButton = new JButton(t("scan!"));
		scanButton.addActionListener(ev -> worker.scan());
		return scanButton;
	}

	public void disableScan() {
		scanButton.setEnabled(false);
	}

	public void enableScan() {
		scanButton.setEnabled(true);
	}
}
